@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2002,2003 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2009,2013 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -30,7 +30,7 @@
  *   Author:  Juergen Pfeifer, 1995,1997                                    *
  ****************************************************************************/
 
-/* $Id: form.h,v 0.17 2003/11/08 20:39:08 tom Exp $ */
+/* $Id: form.h,v 0.23 2013/12/07 17:57:32 tom Exp $ */
 
 #ifndef FORM_H
 #define FORM_H
@@ -42,6 +42,14 @@
   extern "C" {
 #endif
 
+#ifndef FORM_PRIV_H
+typedef void *FIELD_CELL;
+#endif
+
+#ifndef NCURSES_FIELD_INTERNALS
+#define NCURSES_FIELD_INTERNALS /* nothing */
+#endif
+
 typedef int Form_Options;
 typedef int Field_Options;
 
@@ -50,10 +58,10 @@ typedef int Field_Options;
 	**********/
 
 typedef struct {
-  short	pmin;	  /* index of first field on page            */
-  short	pmax;	  /* index of last field on page             */
-  short	smin;	  /* index of top leftmost field on page     */
-  short	smax;	  /* index of bottom rightmost field on page */
+  short pmin;		/* index of first field on page			*/
+  short pmax;		/* index of last field on page			*/
+  short smin;		/* index of top leftmost field on page		*/
+  short smax;		/* index of bottom rightmost field on page	*/
 } _PAGE;
 
 	/**********
@@ -61,86 +69,117 @@ typedef struct {
 	**********/
 
 typedef struct fieldnode {
-  unsigned short       	status;	  /* flags		        */
-  short			rows;	  /* size in rows		*/
-  short			cols;	  /* size in cols		*/
-  short			frow;	  /* first row		        */
-  short			fcol;	  /* first col		        */
-  int                   drows;    /* dynamic rows               */
-  int                   dcols;    /* dynamic cols               */
-  int                   maxgrow;  /* maximum field growth       */
-  int			nrow;	  /* off-screen rows	        */
-  short			nbuf;	  /* additional buffers	        */
-  short			just;	  /* justification	        */
-  short			page;	  /* page on form		*/
-  short			index;	  /* into form -> field	        */
-  int			pad;	  /* pad character	        */
-  chtype		fore;	  /* foreground attribute	*/
-  chtype		back;	  /* background attribute	*/
-  Field_Options		opts;	  /* options		        */
-  struct fieldnode *	snext;	  /* sorted order pointer	*/
-  struct fieldnode *	sprev;	  /* sorted order pointer	*/
-  struct fieldnode *	link;	  /* linked field chain	        */
-  struct formnode *	form;	  /* containing form	        */
-  struct typenode *	type;	  /* field type		        */
-  void *		arg;	  /* argument for type	        */
-  char *		buf;	  /* field buffers	        */
-  void *		usrptr;	  /* user pointer		*/
+  unsigned short	status;		/* flags			*/
+  short			rows;		/* size in rows			*/
+  short			cols;		/* size in cols			*/
+  short			frow;		/* first row			*/
+  short			fcol;		/* first col			*/
+  int			drows;		/* dynamic rows			*/
+  int			dcols;		/* dynamic cols			*/
+  int			maxgrow;	/* maximum field growth		*/
+  int			nrow;		/* off-screen rows		*/
+  short			nbuf;		/* additional buffers		*/
+  short			just;		/* justification		*/
+  short			page;		/* page on form			*/
+  short			index;		/* into form -> field		*/
+  int			pad;		/* pad character		*/
+  chtype		fore;		/* foreground attribute		*/
+  chtype		back;		/* background attribute		*/
+  Field_Options		opts;		/* options			*/
+  struct fieldnode *	snext;		/* sorted order pointer		*/
+  struct fieldnode *	sprev;		/* sorted order pointer		*/
+  struct fieldnode *	link;		/* linked field chain		*/
+  struct formnode *	form;		/* containing form		*/
+  struct typenode *	type;		/* field type			*/
+  void *		arg;		/* argument for type		*/
+  FIELD_CELL *		buf;		/* field buffers		*/
+  void *		usrptr;		/* user pointer			*/
+  /*
+   * The wide-character configuration requires extra information.  Because
+   * there are existing applications that manipulate the members of FIELD
+   * directly, we cannot make the struct opaque.  Offsets of members up to
+   * this point are the same in the narrow- and wide-character configuration.
+   * But note that the type of buf depends on the configuration, and is made
+   * opaque for that reason.
+   */
+  NCURSES_FIELD_INTERNALS
 } FIELD;
 
-	/**************
-	*  FIELDTYPE  *
-	**************/
-
-typedef struct typenode {
-  unsigned short	status;	               /* flags		       */
-  long			ref;	               /* reference count      */
-  struct typenode *	left;	               /* ptr to operand for | */
-  struct typenode *	right;	               /* ptr to operand for | */
-
-  void* (*makearg)(va_list *);                 /* make fieldtype arg   */
-  void* (*copyarg)(const void *);              /* copy fieldtype arg   */
-  void  (*freearg)(void *);                    /* free fieldtype arg   */
-
-  bool	(*fcheck)(FIELD *,const void *);       /* field validation     */
-  bool	(*ccheck)(int,const void *);           /* character validation */
-
-  bool	(*next)(FIELD *,const void *);         /* enumerate next value */
-  bool	(*prev)(FIELD *,const void *);         /* enumerate prev value */
-
-} FIELDTYPE;
 
 	/*********
 	*  FORM  *
 	*********/
 
 typedef struct formnode {
-  unsigned short	status;	  /* flags		        */
-  short			rows;	  /* size in rows		*/
-  short			cols;	  /* size in cols		*/
-  int			currow;	  /* current row in field window*/
-  int			curcol;	  /* current col in field window*/
-  int			toprow;	  /* in scrollable field window	*/
-  int                   begincol; /* in horiz. scrollable field */
-  short			maxfield; /* number of fields	        */
-  short			maxpage;  /* number of pages	        */
-  short			curpage;  /* index into page	        */
-  Form_Options		opts;	  /* options		        */
-  WINDOW *		win;	  /* window		        */
-  WINDOW *		sub;	  /* subwindow		        */
-  WINDOW *		w;	  /* window for current field	*/
-  FIELD **		field;	  /* field [maxfield]	        */
-  FIELD *		current;  /* current field	        */
-  _PAGE *		page;	  /* page [maxpage]	        */
-  void *		usrptr;	  /* user pointer		*/
+  unsigned short	status;	  	/* flags			*/
+  short			rows;		/* size in rows			*/
+  short			cols;		/* size in cols			*/
+  int			currow;		/* current row in field window	*/
+  int			curcol;		/* current col in field window	*/
+  int			toprow;		/* in scrollable field window	*/
+  int			begincol;	/* in horiz. scrollable field	*/
+  short			maxfield;	/* number of fields		*/
+  short			maxpage;	/* number of pages		*/
+  short			curpage;	/* index into page		*/
+  Form_Options		opts;		/* options			*/
+  WINDOW *		win;		/* window			*/
+  WINDOW *		sub;		/* subwindow			*/
+  WINDOW *		w;		/* window for current field	*/
+  FIELD **		field;		/* field [maxfield]		*/
+  FIELD *		current;	/* current field		*/
+  _PAGE *		page;		/* page [maxpage]		*/
+  void *		usrptr;		/* user pointer			*/
 
-  void                  (*forminit)(struct formnode *);
-  void                  (*formterm)(struct formnode *);
-  void                  (*fieldinit)(struct formnode *);
-  void                  (*fieldterm)(struct formnode *);
+  void			(*forminit)(struct formnode *);
+  void			(*formterm)(struct formnode *);
+  void			(*fieldinit)(struct formnode *);
+  void			(*fieldterm)(struct formnode *);
 
 } FORM;
 
+
+	/**************
+	*  FIELDTYPE  *
+	**************/
+
+typedef struct typenode {
+  unsigned short	status;			/* flags		    */
+  long			ref;			/* reference count	    */
+  struct typenode *	left;			/* ptr to operand for |     */
+  struct typenode *	right;			/* ptr to operand for |     */
+
+  void* (*makearg)(va_list *);			/* make fieldtype arg	    */
+  void* (*copyarg)(const void *);		/* copy fieldtype arg 	    */
+  void	(*freearg)(void *);			/* free fieldtype arg	    */
+
+#if NCURSES_INTEROP_FUNCS
+  union {
+    bool (*ofcheck)(FIELD *,const void *);	/* field validation	    */
+    bool (*gfcheck)(FORM*,FIELD *,const void*);	/* generic field validation */
+  } fieldcheck;
+  union {
+    bool (*occheck)(int,const void *);		/* character validation     */
+    bool (*gccheck)(int,FORM*,
+		    FIELD*,const void*);        /* generic char validation  */
+  } charcheck;
+  union {
+    bool (*onext)(FIELD *,const void *);        /* enumerate next value     */
+    bool (*gnext)(FORM*,FIELD*,const void*);    /* generic enumerate next   */
+  } enum_next;
+  union {
+    bool (*oprev)(FIELD *,const void *);	/* enumerate prev value     */
+    bool (*gprev)(FORM*,FIELD*,const void*);    /* generic enumerate prev   */
+  } enum_prev;
+  void* (*genericarg)(void*);                   /* Alternate Arg method     */
+#else
+  bool	(*fcheck)(FIELD *,const void *);	/* field validation	*/
+  bool	(*ccheck)(int,const void *);		/* character validation */
+
+  bool	(*next)(FIELD *,const void *);		/* enumerate next value */
+  bool	(*prev)(FIELD *,const void *);		/* enumerate prev value */
+#endif
+} FIELDTYPE;
+
 typedef void (*Form_Hook)(FORM *);
 
 	/***************************
@@ -154,20 +193,20 @@ typedef void (*Form_Hook)(FORM *);
 #define JUSTIFY_RIGHT		(3)
 
 /* field options */
-#define O_VISIBLE		(0x0001)
-#define O_ACTIVE		(0x0002)
-#define O_PUBLIC		(0x0004)
-#define O_EDIT			(0x0008)
-#define O_WRAP			(0x0010)
-#define O_BLANK			(0x0020)
-#define O_AUTOSKIP		(0x0040)
-#define O_NULLOK		(0x0080)
-#define O_PASSOK		(0x0100)
-#define O_STATIC                (0x0200)
+#define O_VISIBLE		(0x0001U)
+#define O_ACTIVE		(0x0002U)
+#define O_PUBLIC		(0x0004U)
+#define O_EDIT			(0x0008U)
+#define O_WRAP			(0x0010U)
+#define O_BLANK			(0x0020U)
+#define O_AUTOSKIP		(0x0040U)
+#define O_NULLOK		(0x0080U)
+#define O_PASSOK		(0x0100U)
+#define O_STATIC		(0x0200U)
 
 /* form options */
-#define O_NL_OVERLOAD		(0x0001)
-#define O_BS_OVERLOAD		(0x0002)
+#define O_NL_OVERLOAD		(0x0001U)
+#define O_BS_OVERLOAD		(0x0002U)
 
 /* form driver commands */
 #define REQ_NEXT_PAGE	 (KEY_MAX + 1)	/* move to next page		*/
@@ -219,14 +258,14 @@ typedef void (*Form_Hook)(FORM *);
 #define REQ_SCR_BLINE	 (KEY_MAX + 44)	/* scroll field backward a line	*/
 #define REQ_SCR_FPAGE	 (KEY_MAX + 45)	/* scroll field forward a page	*/
 #define REQ_SCR_BPAGE	 (KEY_MAX + 46)	/* scroll field backward a page	*/
-#define REQ_SCR_FHPAGE   (KEY_MAX + 47) /* scroll field forward  half page */
-#define REQ_SCR_BHPAGE   (KEY_MAX + 48) /* scroll field backward half page */
-#define REQ_SCR_FCHAR    (KEY_MAX + 49) /* horizontal scroll char          */
-#define REQ_SCR_BCHAR    (KEY_MAX + 50) /* horizontal scroll char          */
-#define REQ_SCR_HFLINE   (KEY_MAX + 51) /* horizontal scroll line          */
-#define REQ_SCR_HBLINE   (KEY_MAX + 52) /* horizontal scroll line          */
-#define REQ_SCR_HFHALF   (KEY_MAX + 53) /* horizontal scroll half line     */
-#define REQ_SCR_HBHALF   (KEY_MAX + 54) /* horizontal scroll half line     */
+#define REQ_SCR_FHPAGE	 (KEY_MAX + 47) /* scroll field forward	 half page */
+#define REQ_SCR_BHPAGE	 (KEY_MAX + 48) /* scroll field backward half page */
+#define REQ_SCR_FCHAR	 (KEY_MAX + 49) /* horizontal scroll char	*/
+#define REQ_SCR_BCHAR	 (KEY_MAX + 50) /* horizontal scroll char	*/
+#define REQ_SCR_HFLINE	 (KEY_MAX + 51) /* horizontal scroll line	*/
+#define REQ_SCR_HBLINE	 (KEY_MAX + 52) /* horizontal scroll line	*/
+#define REQ_SCR_HFHALF	 (KEY_MAX + 53) /* horizontal scroll half line	*/
+#define REQ_SCR_HBHALF	 (KEY_MAX + 54) /* horizontal scroll half line	*/
 
 #define REQ_VALIDATION	 (KEY_MAX + 55)	/* validate field		*/
 #define REQ_NEXT_CHOICE	 (KEY_MAX + 56)	/* display next field choice	*/
@@ -255,26 +294,20 @@ extern NCURSES_EXPORT_VAR(FIELDTYPE *) TYPE_INTEGER;
 extern NCURSES_EXPORT_VAR(FIELDTYPE *) TYPE_NUMERIC;
 extern NCURSES_EXPORT_VAR(FIELDTYPE *) TYPE_REGEXP;
 
-        /************************************
+	/************************************
 	*  built-in additional field types  *
-        *  They are not defined in SVr4     *
+	*  They are not defined in SVr4     *
 	************************************/
 extern NCURSES_EXPORT_VAR(FIELDTYPE *) TYPE_IPV4;      /* Internet IP Version 4 address */
 
-        /*********************** 
-        *   Default objects    *
-        ***********************/ 
-extern NCURSES_EXPORT_VAR(FORM *)	_nc_Default_Form;
-extern NCURSES_EXPORT_VAR(FIELD *)	_nc_Default_Field;
-
-
 	/***********************
 	*  FIELDTYPE routines  *
 	***********************/
 extern NCURSES_EXPORT(FIELDTYPE *) new_fieldtype (
 		    bool (* const field_check)(FIELD *,const void *),
-		    bool (* const char_check)(int,const void *)),
-                *link_fieldtype(FIELDTYPE *,FIELDTYPE *);
+		    bool (* const char_check)(int,const void *));
+extern NCURSES_EXPORT(FIELDTYPE *) link_fieldtype(
+		    FIELDTYPE *, FIELDTYPE *);
 
 extern NCURSES_EXPORT(int)	free_fieldtype (FIELDTYPE *);
 extern NCURSES_EXPORT(int)	set_fieldtype_arg (FIELDTYPE *,
@@ -326,7 +359,7 @@ extern NCURSES_EXPORT(FIELDTYPE *)	field_type (const FIELD *);
 
 extern NCURSES_EXPORT(char *)	field_buffer (const FIELD *,int);
 
-extern NCURSES_EXPORT(Field_Options)	field_opts (const FIELD *); 
+extern NCURSES_EXPORT(Field_Options)	field_opts (const FIELD *);
 
 	/******************
 	*  FORM routines  *
@@ -363,6 +396,9 @@ extern NCURSES_EXPORT(int)	post_form (FORM *);
 extern NCURSES_EXPORT(int)	unpost_form (FORM *);
 extern NCURSES_EXPORT(int)	pos_form_cursor (FORM *);
 extern NCURSES_EXPORT(int)	form_driver (FORM *,int);
+# if NCURSES_WIDECHAR
+extern NCURSES_EXPORT(int)	form_driver_w (FORM *,int,wchar_t);
+# endif
 extern NCURSES_EXPORT(int)	set_form_userptr (FORM *,void *);
 extern NCURSES_EXPORT(int)	set_form_opts (FORM *,Form_Options);
 extern NCURSES_EXPORT(int)	form_opts_on (FORM *,Form_Options);
@@ -378,6 +414,10 @@ extern NCURSES_EXPORT(Form_Options)	form_opts (const FORM *);
 extern NCURSES_EXPORT(bool)	data_ahead (const FORM *);
 extern NCURSES_EXPORT(bool)	data_behind (const FORM *);
 
+#if NCURSES_SP_FUNCS
+extern NCURSES_EXPORT(FORM *)	NCURSES_SP_NAME(new_form) (SCREEN*, FIELD **);
+#endif
+
 #ifdef __cplusplus
   }
 #endif
