@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
+ * Copyright (c) 1998-2005,2010 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -37,7 +37,7 @@
 
 #include "menu.priv.h"
 
-MODULE_ID("$Id: m_items.c,v 1.12 2003/11/08 20:50:58 tom Exp $")
+MODULE_ID("$Id: m_items.c,v 1.17 2010/01/23 21:20:10 tom Exp $")
 
 /*---------------------------------------------------------------------------
 |   Facility      :  libnmenu  
@@ -53,26 +53,28 @@ MODULE_ID("$Id: m_items.c,v 1.12 2003/11/08 20:50:58 tom Exp $")
 |                                     passed to the function
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(int)
-set_menu_items (MENU * menu, ITEM ** items)
+set_menu_items(MENU * menu, ITEM ** items)
 {
+  T((T_CALLED("set_menu_items(%p,%p)"), (void *)menu, (void *)items));
+
   if (!menu || (items && !(*items)))
     RETURN(E_BAD_ARGUMENT);
-  
-  if ( menu->status & _POSTED )
+
+  if (menu->status & _POSTED)
     RETURN(E_POSTED);
-  
+
   if (menu->items)
     _nc_Disconnect_Items(menu);
-  
+
   if (items)
     {
-      if(!_nc_Connect_Items( menu, items )) 
+      if (!_nc_Connect_Items(menu, items))
 	RETURN(E_CONNECTED);
     }
-  
+
   menu->items = items;
   RETURN(E_OK);
-}		
+}
 
 /*---------------------------------------------------------------------------
 |   Facility      :  libnmenu  
@@ -83,9 +85,10 @@ set_menu_items (MENU * menu, ITEM ** items)
 |   Return Values :  NULL on error
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(ITEM **)
-menu_items (const MENU *menu)
+menu_items(const MENU * menu)
 {
-  return(menu ? menu->items : (ITEM **)0);
+  T((T_CALLED("menu_items(%p)"), (const void *)menu));
+  returnItemPtr(menu ? menu->items : (ITEM **) 0);
 }
 
 /*---------------------------------------------------------------------------
@@ -98,9 +101,10 @@ menu_items (const MENU *menu)
 |   Return Values :  Number of items or -1 to indicate error.
 +--------------------------------------------------------------------------*/
 NCURSES_EXPORT(int)
-item_count (const MENU *menu)
+item_count(const MENU * menu)
 {
-  return(menu ? menu->nitems : -1);
+  T((T_CALLED("item_count(%p)"), (const void *)menu));
+  returnCode(menu ? menu->nitems : -1);
 }
 
 /* m_items.c ends here */
